from odoo import _, models, fields, api
from odoo.exceptions import ValidationError

from odoo.addons.base_iban.models.res_partner_bank import \
    normalize_iban, pretty_iban, _map_iban_template


class CrmLead(models.Model):
    _inherit = 'crm.lead'
    subscription_request_id = fields.Many2one(
        'subscription.request', 'Subscription Request'
    )
    iban = fields.Char(string="IBAN")

    mobile_lead_line_id = fields.Many2one(
        'crm.lead.line',
        compute='_compute_mobile_lead_line_id',
        string="Mobile Lead Line",
    )

    skip_duplicated_phone_validation = fields.Boolean(
        string="Skip duplicated phone validation"
    )

    # TODO: To modify if in the future we can have more than one `mobile_lead_line_id`
    def _compute_mobile_lead_line_id(self):
        for crm_lead in self:
            for line in crm_lead.lead_line_ids:
                if line.mobile_isp_info:
                    crm_lead.mobile_lead_line_id = line
                    break

    def _ensure_crm_lead_iban_belongs_to_partner(self, crm_lead):
        partner_bank_ids = crm_lead.partner_id.bank_ids
        partner_iban_list = [bank.sanitized_acc_number for bank in partner_bank_ids]

        if crm_lead.iban and crm_lead.iban not in partner_iban_list:
            self.env['res.partner.bank'].create({
                'acc_type': 'iban',
                'acc_number': crm_lead.iban,
                'partner_id': crm_lead.partner_id.id
            })

    def action_set_won(self):
        for crm_lead in self:
            crm_lead.validate_won()
            if crm_lead.iban:
                self._ensure_crm_lead_iban_belongs_to_partner(crm_lead)
        super(CrmLead, self).action_set_won()

    def validate_won(self):
        if self.stage_id != self.env.ref("crm.stage_lead3"):
            raise ValidationError(
                _("The crm lead must be in remesa stage.")
            )
        if len(self.lead_line_ids) > 1:
            raise ValidationError(_(
                "The CRMLead to validate has more than one CRMLeadLine associated."
                " This shouldn't happen. Please contact the IP team."
            ))

    def _get_email_from_partner_or_SR(self, vals):
        if vals.get('partner_id'):
            contact_id = vals.get('partner_id')
            model = self.env['res.partner']
        else:
            contact_id = vals.get('subscription_request_id')
            model = self.env['subscription.request']
        return model.browse(contact_id).email

    @api.model
    def create(self, vals):
        if not vals.get("email_from"):
            vals["email_from"] = self._get_email_from_partner_or_SR(vals)
        return super(CrmLead, self).create(vals)

    def action_set_paused(self):
        paused_stage_id = self.env.ref('crm.stage_lead2').id
        for crm_lead in self:
            crm_lead.write({'stage_id': paused_stage_id})

    def action_set_remesa(self):
        remesa_stage_id = self.env.ref('crm.stage_lead3').id
        for crm_lead in self:
            crm_lead.validate_remesa()
            crm_lead.write({'stage_id': remesa_stage_id})

    def validate_remesa(self):
        self.ensure_one()
        # Check if related SR is validated
        if not self.partner_id:
            raise ValidationError(
                _("The subscription request related must be validated.")
            )
        # Validate IBAN
        if not self._get_bank_from_iban():
            raise ValidationError(
                _("Invalid bank.")
            )
        # Validate phone number
        if self._phones_already_used():
            raise ValidationError(
                _("Contract or validated CRMLead with the same phone already exists.")  # noqa
            )

    def _get_bank_from_iban(self):
        self.ensure_one()
        # Code copied from base_bank_from_iban module:
        # https://github.com/OCA/community-data-files/blob/12.0/base_bank_from_iban/models/res_partner_bank.py#L13  # noqa
        acc_number = pretty_iban(normalize_iban(self.iban)).upper()
        country_code = acc_number[:2].lower()
        iban_template = _map_iban_template[country_code]
        first_match = iban_template[2:].find('B') + 2
        last_match = iban_template.rfind('B') + 1
        bank_code = acc_number[first_match:last_match].replace(' ', '')
        bank = self.env['res.bank'].search([
            ('code', '=', bank_code),
            ('country.code', '=', country_code.upper()),
        ], limit=1)
        return bank

    def _phones_already_used(self):
        self.ensure_one()
        if self.skip_duplicated_phone_validation:
            return False

        for line in self.lead_line_ids:
            phone = False
            if line.mobile_isp_info:
                phone = line.mobile_isp_info.phone_number
            else:
                phone = line.broadband_isp_info.phone_number
            if not phone:
                return False
            contracts = self.env["contract.contract"].search([
                "|",
                "|",
                "|",
                ("mobile_contract_service_info_id.phone_number", "=", phone),
                ("vodafone_fiber_service_contract_info_id.phone_number", "=", phone),
                ("mm_fiber_service_contract_info_id.phone_number", "=", phone),
                ("adsl_service_contract_info_id.phone_number", "=", phone),
            ])
            won_stage_id = self.env.ref("crm.stage_lead4").id
            order_lines = self.env["crm.lead.line"].search([
                ("lead_id.stage_id", "=", won_stage_id),
                "|",
                ("mobile_isp_info.phone_number", "=", phone),
                ("broadband_isp_info.phone_number", "=", phone),
            ])
            if contracts or order_lines:
                return True
